package com.gic.enterprise.service.outer.impl;
import com.alibaba.druid.sql.SQLUtils;
import com.alibaba.druid.sql.ast.SQLExpr;
import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.*;
import com.gic.enterprise.dto.DownloadReportDTO;
import com.gic.enterprise.dto.HiveDataDownloadDTO;
import com.gic.enterprise.dto.HiveQueryCondition;
import com.gic.enterprise.entity.TabDownloadReport;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.pojo.DownloadTaskPOJO;
import com.gic.enterprise.service.DownloadReportApiService;
import com.gic.enterprise.service.DownloadReportService;
import com.gic.enterprise.service.HiveDataDownloadApiService;
import com.gic.enterprise.utils.FileUtils;
import com.gic.enterprise.utils.HiveConnUtils;
import com.gic.redis.data.util.RedisUtil;
import com.gic.store.utils.valid.ValidUtil;
import com.gic.thirdparty.BucketNameEnum;
import com.gic.thirdparty.FileOperateUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RSet;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;

import java.io.File;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.*;

import java.util.Date;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Service("hiveDataDownloadApiService")
public class HiveDataDownloadApiServiceImpl implements HiveDataDownloadApiService {
    private static final Logger log = LogManager.getLogger(HiveDataDownloadApiServiceImpl.class);

    ScheduledExecutorService downloadService = new ScheduledThreadPoolExecutor(1,
            new BasicThreadFactory.Builder().namingPattern("downloadTimer-%d").daemon(true).build());
    ScheduledExecutorService applyService = new ScheduledThreadPoolExecutor(1,
            new BasicThreadFactory.Builder().namingPattern("applyTimer-%d").daemon(true).build());

    private List<DownloadTaskPOJO> taskList = new ArrayList<>();

    @Autowired
    private DownloadReportApiService downloadReportApiService;
    @Autowired
    private DownloadReportService downloadReportService;

    @PostConstruct
    private void init(){
        //启动下载定时任务
        runDownloadTask(3);
        //启动获取任务审核状态定时任务
        runApplyTask(5);
        //初始化任务
        initTask();
    }

    /***
     * 1 创建下载记录
     *    如果需要审批  发送审批申请
     *    需要定时拉去任务的审核状态
     * 2 写入任务队列
     * 3 定时处理任务队列
     * @param hiveDataDownloadDTO
     * @return
     */

    @Override
    public ServiceResponse<Integer> createDownloadTask(HiveDataDownloadDTO hiveDataDownloadDTO) {
        //校验参数
        ServiceResponse paramResult = ValidUtil.allCheckValidate(hiveDataDownloadDTO, HiveDataDownloadDTO.hiveData.class);
        if(!paramResult.isSuccess()){
            return paramResult;
        }
        DownloadReportDTO downloadReportDTO = EntityUtil.changeEntityByJSON(DownloadReportDTO.class, hiveDataDownloadDTO);
        downloadReportDTO.setDataCount(0);
        String sql = "";
        DownloadTaskPOJO downloadTaskPOJO = null;
        try{
            sql = getSql(hiveDataDownloadDTO);
            downloadTaskPOJO = buildTask(hiveDataDownloadDTO, sql);
            downloadReportDTO.setHiveSql(JSON.toJSONString(downloadTaskPOJO));
        }catch (Exception e){
            log.warn(e);
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), e.getMessage());
        }

        downloadReportDTO.setDataSource("hive");
        ServiceResponse<Integer> response = downloadReportApiService.saveDownloadReport(downloadReportDTO);
        if(!response.isSuccess()){
            return response;
        }
        downloadTaskPOJO.setReportId(response.getResult());
        taskList.add(downloadTaskPOJO);
        return response;
    }

    private DownloadTaskPOJO buildTask(HiveDataDownloadDTO hiveDataDownloadDTO, String sql){
        DownloadTaskPOJO downloadTaskPOJO = new DownloadTaskPOJO();
        downloadTaskPOJO.setHiveSql(sql);
        downloadTaskPOJO.setCanDownload(!DownloadReportDataTypeEnum.COMPLETE_DATA.getCode().equals(hiveDataDownloadDTO.getDataType()));
        downloadTaskPOJO.setDesenField(hiveDataDownloadDTO.getDesenField());
        downloadTaskPOJO.setExcelExtension(hiveDataDownloadDTO.getExcelExtension());
        downloadTaskPOJO.setTableName(hiveDataDownloadDTO.getTableName());
        downloadTaskPOJO.setFileName(hiveDataDownloadDTO.getFileName());
        return downloadTaskPOJO;
    }

    private void initTask(){
        List<TabDownloadReport> list = downloadReportService.listUnfinishedTask();
        for(TabDownloadReport report : list){
            DownloadTaskPOJO downloadTaskPOJO = JSON.parseObject(report.getHiveSql(), DownloadTaskPOJO.class);
            taskList.add(downloadTaskPOJO);
        }
    }


    private void runDownloadTask(Integer interval) {
        downloadService.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                try{
                    Iterator<DownloadTaskPOJO> iterator = taskList.iterator();
                    log.info("定时执行任务下载：{}", taskList.size());
                    while (iterator.hasNext()){
                        DownloadTaskPOJO task = iterator.next();
                        boolean exist = RedisUtil.getRedisClient().getBucket("report:" + task.getReportId()).trySet(task.getReportId());
                        if(exist){
                            continue;
                        }
                        if(task.isCanDownload()){
                            log.info("开始下载任务:{}", task.getReportId());
                            iterator.remove();
                            // 修改任务状态为生成中
                            downloadReportService.updateDownloadStatus(task.getReportId(), DownloadReportStatusEnum.BUILD);
                            // 获取hive连接
                            Connection conn = HiveConnUtils.getConnection();
                            Statement stat = null;
                            try {
                                stat = conn.createStatement();
                                stat.execute("REFRESH TABLE " + task.getTableName()); // 强制刷新表结构  防止数据出错
                                ResultSet rs = stat.executeQuery(task.getHiveSql());
                                String url = FileUtils.createFile(rs, task.getDataType(), task.getDesenField(), task.getFileName(), task.getExcelExtension());
                                downloadReportService.updateDownloadUrl(Integer.valueOf(task.getReportId()), url, rs.getRow());
                            } catch (Exception e) {
                                log.warn("下载数据出现异常", e);
                                // 任务状态修改
                                downloadReportService.updateDownloadStatus(task.getReportId(), DownloadReportStatusEnum.ERROR);
                            } finally {
                                try {
                                    if(stat != null) {
                                        stat.close();
                                    }
                                    if(conn != null) {
                                        conn.close();
                                    }
                                } catch (SQLException e) {
                                    log.warn("stat 关闭异常", e);
                                }
                            }
                        }
                    }
                }catch (Throwable e){
                    log.warn(e);
                }
            }
        }, interval*1000, interval*1000, TimeUnit.MILLISECONDS);
    }

    private void runApplyTask(Integer interval) {
        applyService.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                try{
                    Iterator<DownloadTaskPOJO> iterator = taskList.iterator();
                    while (iterator.hasNext()){
                        DownloadTaskPOJO task = iterator.next();
                        if(!task.isCanDownload()){
                            TabDownloadReport report = downloadReportService.getById(Integer.valueOf(task.getReportId()));
                            Integer auditResult = report.getAuditResult();
                            /**没有审批通过则直接跳过*/
                            if(DownloadReportStatusEnum.WAIT.getCode().equals(auditResult)){
                                continue;
                            }
                            /**审批通过 设置任务为可下载状态***/
                            if(DownloadReportStatusEnum.PASS.getCode().equals(auditResult)){
                                task.setCanDownload(true);
                            }
                            /**超时移除任务*/
                            if(DownloadReportStatusEnum.OVER_TIME.getCode().equals(auditResult)){
                                iterator.remove();
                            }
                        }

                    }
                }catch (Throwable e){
                    log.warn(e);
                }
            }
        }, interval*1000, interval*1000, TimeUnit.MILLISECONDS);
    }

    public String getSql(HiveDataDownloadDTO hiveDataDownloadDTO) throws Exception {
        List<HiveQueryCondition> queryConditionList = hiveDataDownloadDTO.getQueryConditionList();
        String tableName = hiveDataDownloadDTO.getTableName();
        StringBuffer sql = new StringBuffer(" select ");
        int index = 1;
        for(HiveQueryCondition condition: queryConditionList){
            sql.append(condition.getFieldName()).append(" as `").append(condition.getFieldMark()).append("`");
            if(index < queryConditionList.size()){
                sql.append(",");
            }
            index++;
        }
        sql.append(" from ").append(tableName);
        sql.append(" where 1=1 ");
        boolean hasEnterpriseIdFilter = false;
        for(HiveQueryCondition condition: queryConditionList){
            if(condition.getFieldFilterOptEnum() != null){
                if(condition.getFieldName().equals("enterprise_id") || condition.getFieldName().equals("ent_id")){
                    hasEnterpriseIdFilter = true;
                }
                sql.append(" and ").append(condition.getFieldName()).append(" ").append(condition.getFieldFilterOptEnum().getOpt());
                if(FieldFilterOptEnum.IN.equals(condition.getFieldFilterOptEnum()) ||
                        FieldFilterOptEnum.NOTIN.equals(condition.getFieldFilterOptEnum())){
                    sql.append("(");
                    if(FieldTypeEnum.DATE.equals(condition.getFieldTypeEnum()) || FieldTypeEnum.STRING.equals(condition.getFieldTypeEnum())){
                        sql.append("'").append(condition.getFilterValue().get(0)).append("'");
                    } else {
                        sql.append(condition.getFilterValue().get(0));
                    }
                    sql.append(")");
                } else {
                    if(FieldTypeEnum.DATE.equals(condition.getFieldTypeEnum()) || FieldTypeEnum.STRING.equals(condition.getFieldTypeEnum())){
                        sql.append("'").append(condition.getFilterValue().get(0)).append("'");
                    } else {
                        sql.append(condition.getFilterValue().get(0));
                    }
                }
            }
        }
        log.info("sql:{}", sql.toString());
        if(!hasEnterpriseIdFilter){
            throw new Exception("查询条件中必须要有企业id");
        }
        return sql.toString();
    }

}
