package com.gic.enterprise.service.outer.impl.paycomplete;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dto.ApprovalCallBackDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.entity.TabBillingPayInfo;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.service.BillingRechargeRecordService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.log.api.dto.SystemSetLogDTO;
import com.gic.log.api.service.LogApiService;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 余额充值支付完成
 * @Description:
 * @author zhiwj
 * @date 2020-11-24 15:57
 */
@Component("BALANCE_RECHARGE")
public class RechargeHandler implements PayCompleteHandler {

    private Logger logger = LogManager.getLogger(RechargeHandler.class);

    @Autowired
    private BillingRechargeRecordService billingRechargeRecordService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private LogApiService logApiService;

    @Override
    public void payComplete(ApprovalCallBackDTO resData, TabBillingPayInfo billingPayInfo) {
        // 通知
        pushMessageApiService.pushMessage(resData.getEnterpriseId(), "系统通知", "余额充值", "您的商户成功充值"+resData.getTotalFeePaid()+"元！感谢您的支持和使用！");
        // 充值成功短信
        sendRechargeSuccessSms(resData, billingPayInfo.getEnterpriseId());
        // 日志
        TabBillingRechargeRecord rechargeRecord = billingRechargeRecordService.getRechargeRecordBySerialNumber(billingPayInfo.getOrderSerialNumber());
        UserDTO userDTO = userApiService.getUserInfoById(rechargeRecord.getInitiatorUser()).getResult();
        doLog("充值", "账户余额", userDTO);
        // 审核成功短信
        if (PayTypeEnum.OFFLINE_PAY.getCode() == resData.getPayType()) {
            sendApprovalSms(resData, "GICJFZX008");
        }
        billingRechargeRecordService.updateBySerialNumber(resData.getEnterpriseId(), resData.getOrderSerialNumber(),
                resData.getTotalFee(), resData.getTotalFeePaid(), resData.getAuditStatus(), resData.getTimeEnd(), resData.getOnlineSerialNumber());
    }

    private void sendApprovalSms(ApprovalCallBackDTO resData, String code) {
        TabBillingRechargeRecord rechargeRecord = billingRechargeRecordService.getRechargeRecordBySerialNumber(resData.getOrderSerialNumber());
        Integer initiator = rechargeRecord.getInitiator();
        UserDTO userDTO = userApiService.getUserInfoById(initiator).getResult();
        sendSms(resData, userDTO, code);
    }

    private void sendRechargeSuccessSms(ApprovalCallBackDTO resData, Integer enterpriseId) {
        UserDTO userDTO = userApiService.getUserByEnterpriseId(enterpriseId).getResult();
        sendSms(resData, userDTO, "GICJFZX005");
    }

    private void sendSms(ApprovalCallBackDTO resData, UserDTO userDTO, String code) {
        try {
            if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                userDTO.setPhoneAreaCode("+86");
            }
            EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
            String[] smsArr = new String[]{enterpriseDTO.getEnterpriseName(), resData.getOrderSerialNumber(), BillingAuditStatusEnum.PASS.getCode().equals(resData.getAuditStatus()) ? "成功" : "失败"};
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(code,
                    userDTO.getEnterpriseId(), userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
            if (!smsSendResult.isSuccess()) {
                logger.warn(smsSendResult.getMessage());
            }
        } catch (Exception e) {
            logger.info("发送短信失败", e);
        }
    }

    private void doLog(String content, String operationObject, UserDTO userDTO) {
        final SystemSetLogDTO logDTO = new SystemSetLogDTO();
        //操作时间
        logDTO.setLogTime(new Date());
        logDTO.setCreateTime(new Date());
        //操作人
        logDTO.setUserId(Long.valueOf(userDTO.getUserId()));
        logDTO.setUserName(userDTO.getUserName());
        //操作内容
        logDTO.setContent(content);
        //为了search查询，暂时用这个字段存
        logDTO.setInterfaceName(content);
        ////操作对象
        logDTO.setBusinessName(operationObject);
        //操作模块
        logDTO.setRelationId(45000L);
        logDTO.setProject("企业-计费中心");
        //            }
        //账号（手机号）
        logDTO.setRemark1(userDTO.getPhoneAreaCode());
        logDTO.setRemark2(userDTO.getPhoneNumber());

        logDTO.setEnterpriseId(userDTO.getEnterpriseId().longValue());
        EnterpriseDTO enterpriseDTO = enterpriseApiService.getEnterpriseById(userDTO.getEnterpriseId()).getResult();
        logDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
        logApiService.saveSystemSetLog(logDTO);
    }

}
