package com.gic.enterprise.service.outer.impl.union;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;

import com.alibaba.fastjson.JSON;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResTypeEnum;
import com.gic.enterprise.utils.ExecutorPoolSingleton;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;
import com.gic.member.config.api.service.AuthorizationEnterConfApiService;
import com.gic.member.config.api.service.UserGroupPermissionApiService;
import com.gic.platform.dto.TransmitDataDTO;
import com.gic.platform.manager.TransmitDataHolder;
import com.gic.popup.config.InitPopupConfig;
import com.gic.popup.config.PopupUtil;
import com.gic.popup.dto.PopupDTO;
import com.gic.popup.dto.RelationBusinessInfoDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Import;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.validation.annotation.Validated;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthDetailStatusEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthResOperateEnum;
import com.gic.enterprise.constant.union.UnionEnterpriseAuthStatusEnum;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.union.*;
import com.gic.enterprise.entity.TabEnterprise;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuth;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthApp;
import com.gic.enterprise.entity.TabSysUnionEnterpriseAuthResDetail;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.exception.CommonException;
import com.gic.enterprise.qo.union.UnionEnterpriseAuthResLogQO;
import com.gic.enterprise.service.*;
import com.gic.store.service.StoreAuthorizationApiService;


/**
 * 联合商户
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/2/3 1:52 PM 
 */
@Service("unionEnterpriseAuthApiService")
@Import(value = InitPopupConfig.class)
public class UnionEnterpriseAuthApiServiceImpl implements UnionEnterpriseAuthApiService{
    private static Logger LOGGER = LogManager.getLogger(UnionEnterpriseAuthApiServiceImpl.class);

    /**
     * 联合商户引用类型
     */
    private static final Integer REFER_UNION_ENTERPRISE = 11;

    /**
     * 资源组引用
     */
    private static final Integer REFER_RESOURCE = 5;

    @Autowired
    private UnionEnterpriseAuthService unionEnterpriseAuthService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private UnionEnterpriseAuthResDetailService unionEnterpriseAuthResDetailService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private UnionEnterpriseAuthAppService unionEnterpriseAuthAppService;
    @Autowired
    private UnionEnterpriseAuthResLogService unionEnterpriseAuthResLogService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;
    @Autowired
    private AuthorizationEnterConfApiService authorizationEnterConfApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private UserGroupPermissionApiService userGroupPermissionApiService;
    @Autowired
    private EnterpriseLicensePatternApiService enterpriseLicensePatternApiService;

    /**
     * 待解除中短信模板
     */
    private static String SMS_CODE_RELIEVE = "GICSHHT009";
    /**
     * 解除成功短信模板
     */
    private static String SMS_CODE_RELIEVED = "GICSHHT010";

    @Override
    public ServiceResponse<Void> saveUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        Assert.notNull(ownEnterpriseId, "自有商户不能为空");
        Assert.notNull(unionEnterpriseId, "联合商户不能为空");
        if (ownEnterpriseId.equals(unionEnterpriseId)) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "不能联合自有商户");
        }
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old != null) {
            return ServiceResponse.success();
        }
        unionEnterpriseAuthService.saveUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        return ServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> authResource(@Validated UnionEnterpriseAuthResDTO dto) {
        TabEnterprise tabEnterprise = enterpriseService.getEnterpriseById(dto.getUnionEnterpriseId());
        if (tabEnterprise.getServiceStatus() == 0) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户无服务");
        }
        if (tabEnterprise.getServiceStatus() == 2) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户已停用");
        }

        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(dto.getOwnEnterpriseId(), dto.getUnionEnterpriseId());
        //授权数据
        UnionEnterpriseAuthDTO data = EntityUtil.changeEntityNew(UnionEnterpriseAuthDTO.class, dto);
        data.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());

        //同一个资源组只能授权给一个商户
        List<TabSysUnionEnterpriseAuth> listByResourceGroup = unionEnterpriseAuthService.listUnionEnterpriseAuthByResourceGroupId(dto.getResourceGroupId());
        if (CollectionUtils.isNotEmpty(listByResourceGroup)) {
            if (listByResourceGroup.stream().filter(e -> !e.getUnionEnterpriseAuthId().equals(old.getUnionEnterpriseAuthId())).count() > 0) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组不能授权给不同商户");
            }
        }
        Integer resourceGroupId = dto.getResourceGroupId();

        if (!enterpriseLicensePatternApiService.validUserLicenseEnough(resourceGroupId, dto.getUnionEnterpriseId()).getResult()) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户的用户域license不足");
        }

        //是否是编辑。编辑和新增授权同步资源处理不同
        boolean isEdit = old.getResourceGroupId() != null && !old.getResourceGroupId().equals(dto.getResourceGroupId());
        ResourceGroupDTO resourceGroupDTO = getResourceGroup(resourceGroupId);
        ResourceGroupDTO oldResourceGroupDTO = null;
        //1 : 初始化 2：资源组变更
        Integer resourceGroupChangeType = 1;
        if (isEdit) {
            oldResourceGroupDTO = getResourceGroup(old.getResourceGroupId());
            resourceGroupChangeType = 2;
        }

        //
        checkResourceDealing((old.getUnionEnterpriseAuthId()));

        saveRefer(old.getUnionEnterpriseAuthId(), tabEnterprise.getEnterpriseName(), resourceGroupId);

        unionEnterpriseAuthService.update(data);
        //初始化未授权资源数据
        unionEnterpriseAuthResDetailService.initUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId());

        //授权通知(用户域)
        TransmitDataDTO transmitDataDTO = TransmitDataHolder.get();
        resourceGroupApiService.syncResourceAuthChangeMQ(dto.getOwnEnterpriseId(), dto.getUnionEnterpriseId(), resourceGroupDTO, oldResourceGroupDTO, resourceGroupChangeType, JSON.toJSONString(transmitDataDTO));

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UnionEnterpriseAuthDTO> getUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户不存在");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(UnionEnterpriseAuthDTO.class, old));
    }

    @Override
    public ServiceResponse<Integer> getEnterpriseIdByAuthAccountGroupId(Integer accountGroupId) {
        return ServiceResponse.success(unionEnterpriseAuthService.getEnterpriseIdByAuthAccountGroupId(accountGroupId));
    }

    @Override
    public ServiceResponse<Void> resetAuthResource(Integer ownEnterpriseId, Integer unionEnterpriseId, Integer resourceType) {
        Assert.notNull(ownEnterpriseId, "自有商户不能为空");
        Assert.notNull(unionEnterpriseId, "联合商户不能为空");
        Assert.notNull(resourceType, "资源类型不能为空");

        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);

        TabSysUnionEnterpriseAuthResDetail detail = unionEnterpriseAuthResDetailService
                .getUnionEnterpriseAuthResDetail(old.getUnionEnterpriseAuthId(), resourceType);
        if (detail == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源不存在");
        }
        //todo 重新授权
        //门店资源
        if (UnionEnterpriseAuthResTypeEnum.STORE.getCode() == resourceType) {
            String authKey = detail.getAuthKey();
            storeAuthorizationApiService.reAuthStore(authKey);
        }
        //日志
        unionEnterpriseAuthResLogService.saveUnionEnterpriseAuthResLog(old.getUnionEnterpriseAuthId(),
                resourceType, UnionEnterpriseAuthResOperateEnum.RE_AUTH.getMessage());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> resetRelieveResource(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId, @NotNull Integer resourceType) {
        //todo 目前只有用户域会有解除失败
        if (UnionEnterpriseAuthResTypeEnum.USER_AREA.getCode().equals(resourceType)) {
            TransmitDataDTO transmitDataDTO = TransmitDataHolder.get();
            authorizationEnterConfApiService.delAuthorizationEnterpriseConf(unionEnterpriseId, ownEnterpriseId, transmitDataDTO);
        } else if (UnionEnterpriseAuthResTypeEnum.STORE.getCode().equals(resourceType)) {
            storeAuthorizationApiService.cancelStoreAll(unionEnterpriseId, ownEnterpriseId);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveUnion(Integer ownEnterpriseId, Integer unionEnterpriseId, String relieveReason) {
        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);

        UnionEnterpriseAuthDTO dto = new UnionEnterpriseAuthDTO();
        dto.setRelieveTime(new Date());
        //解除生效时间，保留X个自然日后开始解除资源
        dto.setRelieveEffectDate(7);
        dto.setRelieveRemark(relieveReason);
        dto.setUnionEnterpriseAuthId(old.getUnionEnterpriseAuthId());
        dto.setStatusFlag(UnionEnterpriseAuthStatusEnum.TO_DO_RELIEVE.getCode());

        TransmitDataDTO transmitDataDTO = TransmitDataHolder.get();
        dto.setLogParams(JSON.toJSONString(transmitDataDTO));

        unionEnterpriseAuthService.update(dto);
        //资源状态
        unionEnterpriseAuthResDetailService.relieveUnion(old.getUnionEnterpriseAuthId());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthListDTO>> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId, String enterpriseName, Integer authStatus) {
        //联合商户名称匹配范围
        List<Integer> unionEnterpriseIdList = new ArrayList<>();
        if (StringUtils.isNotBlank(enterpriseName)) {
            List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterpriseByName(enterpriseName);
            if (CollectionUtils.isNotEmpty(enterpriseList)) {
                unionEnterpriseIdList = enterpriseList
                        .stream()
                        .filter(e -> !e.getEnterpriseId().equals(ownEnterpriseId))
                        .mapToInt(e -> e.getEnterpriseId())
                        .boxed()
                        .collect(Collectors.toList());
            }
        }

        List<UnionEnterpriseAuthListDTO> result = unionEnterpriseAuthService.listUnionEnterpriseAuth(ownEnterpriseId, unionEnterpriseIdList, authStatus);
        if (CollectionUtils.isNotEmpty(result)) {
            //组装数据
            //1:联合商户数据
            List<Integer> enterpriseIdList = result.stream().mapToInt(e -> e.getEnterpriseId()).boxed().collect(Collectors.toList());
            Map<Integer, TabEnterprise> enterpriseMap = getEnterprise(enterpriseIdList);

            //2:超管数据
            Map<Integer, UserDTO> userMap = getUser(enterpriseIdList);

            //3:资源明细
            List<Integer> unionEnterpriseAuthIdList = result.stream().mapToInt(e -> e.getUnionEnterpriseAuthId()).boxed().collect(Collectors.toList());
            List<TabSysUnionEnterpriseAuthResDetail> list = unionEnterpriseAuthResDetailService.listByUnionEnterpriseAuthIdList(unionEnterpriseAuthIdList);
            Map<Integer, List<TabSysUnionEnterpriseAuthResDetail>> unionEnterpriseAuthResMap = list
                    .stream()
                    .filter(e -> {
                        if (authStatus != null) {
                            if (authStatus.equals(e.getAuthStatus())) {
                                return true;
                            } else {
                                return false;
                            }
                        } else {
                            return true;
                        }
                    })
                    .collect(Collectors.groupingBy(TabSysUnionEnterpriseAuthResDetail::getUnionEnterpriseAuthId));

            //填充数据
            for (UnionEnterpriseAuthListDTO obj : result) {
                TabEnterprise enterprise = enterpriseMap.get(obj.getEnterpriseId());
                if (enterprise != null) {
                    obj.setCompanyName(enterprise.getCompanyName());
                    obj.setAddress(enterprise.getAddress());
                    obj.setEnterpriseName(enterprise.getEnterpriseName());
                }
                UserDTO user = userMap.get(obj.getEnterpriseId());
                if (user != null) {
                    obj.setUserName(user.getUserName());
                    obj.setPhoneAreaCode(user.getPhoneAreaCode());
                    obj.setPhoneNumber(user.getPhoneNumber());
                }
                List<TabSysUnionEnterpriseAuthResDetail> tempDetail = unionEnterpriseAuthResMap.get(obj.getUnionEnterpriseAuthId());
                obj.setUnionEnterpriseResourceList(EntityUtil.changeEntityListNew(UnionEnterpriseAuthResDetailDTO.class, tempDetail));
            }
            return ServiceResponse.success(result);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthDTO>> listUnionEnterpriseAuth(@NotNull Integer ownEnterpriseId) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listUnionEnterpriseAuth(ownEnterpriseId);
        if (list == null) {
            return ServiceResponse.success(Collections.emptyList());
        }
        return ServiceResponse.success(EntityUtil.changeEntityListNew(UnionEnterpriseAuthDTO.class, list));
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthDTO>> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listOwnEnterpriseAuth(unionEnterpriseId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(UnionEnterpriseAuthDTO.class, list));
    }

    @Override
    public ServiceResponse<Map<Integer, List<Integer>>> getAccountGroupOfOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listOwnEnterpriseAuth(unionEnterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
             return ServiceResponse.success(list.stream()
                     .filter(e -> StringUtils.isNotBlank(e.getAccountGroup()))
                     .collect(Collectors.toMap(e -> e.getOwnEnterpriseId(), e -> {
                 String[] accountGroupArr = e.getAccountGroup().split(",");
                 return Arrays.stream(accountGroupArr).mapToInt(accountGroup -> Integer.parseInt(accountGroup)).boxed().collect(Collectors.toList());
            })));
        }
        return ServiceResponse.success(new HashMap<>());
    }

    @Override
    public ServiceResponse<UnionEnterpriseAuthDTO> getUnionEnterpriseAuthByResourceGroupId(@NotNull Integer resourceGroupId) {
        List<TabSysUnionEnterpriseAuth> listByResourceGroup = unionEnterpriseAuthService.listUnionEnterpriseAuthByResourceGroupId(resourceGroupId);
        if (CollectionUtils.isNotEmpty(listByResourceGroup)) {
            return ServiceResponse.success(EntityUtil.changeEntityNew(UnionEnterpriseAuthDTO.class, listByResourceGroup.get(0)));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveOrUpdateUnionEnterpriseAuthApp(@Valid UnionEnterpriseAuthAppDTO dto) {
        Integer unionEnterpriseAuthAppId = dto.getUnionEnterpriseAuthAppId();

        List<TabSysUnionEnterpriseAuthApp> appList = unionEnterpriseAuthAppService.listUnionEnterpriseAuthApp(dto.getUnionEnterpriseAuthId());
        Set<String> appIdSet = new HashSet<>();
        if (CollectionUtils.isNotEmpty(appList)) {
             appList.forEach(e -> {
                 //编辑的时候去除本条数据的应用ID，避免校验错误
                 if (!e.getUnionEnterpriseAuthAppId().equals(unionEnterpriseAuthAppId)) {
                     String[] appIdArr = e.getAppId().split(",");
                     for (String appId : appIdArr) {
                         if (StringUtils.isBlank(appId)) {
                             continue;
                         }
                         appIdSet.add(appId);
                     }
                 }
            });
        }

        String appIdParam = dto.getAppId();
        String[] appIdParamArr = appIdParam.split(",");
        for (String temp : appIdParamArr) {
            if (StringUtils.isBlank(temp)) {
                continue;
            }
            if (appIdSet.contains(temp)) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "应用已经配置策略");
            }
        }

        if (unionEnterpriseAuthAppService.isRepeatStrategyName(unionEnterpriseAuthAppId, dto.getUnionEnterpriseAuthId(), dto.getStrategyName())) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "策略名称重复");
        }

        if (unionEnterpriseAuthAppId == null) {
            unionEnterpriseAuthAppService.saveUnionEnterpriseAuthApp(dto);
        } else {
            unionEnterpriseAuthAppService.editUnionEnterpriseAuthApp(dto);
        }

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UnionEnterpriseAuthAppDTO> getUnionEnterpriseAuthApp(@NotNull Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = checkUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        return ServiceResponse.success(EntityUtil.changeEntityNew(UnionEnterpriseAuthAppDTO.class, old));
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseAuthAppDTO>> listUnionEnterpriseAuthApp(@NotNull Integer ownEnterpriseId, @NotNull Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = checkExistUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        List<TabSysUnionEnterpriseAuthApp> list = unionEnterpriseAuthAppService.listUnionEnterpriseAuthApp(old.getUnionEnterpriseAuthId());
        if (CollectionUtils.isNotEmpty(list)) {
            return ServiceResponse.success(EntityUtil.changeEntityListNew(UnionEnterpriseAuthAppDTO.class, list));
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<String> deleteUnionEnterpriseAuthApp(@NotNull Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = checkUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        unionEnterpriseAuthAppService.deleteUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        return ServiceResponse.success(old.getStrategyName());
    }

    @Override
    public ServiceResponse<String> deleteUnionEnterpriseAuthAppByUnionEnterpriseAuthId(@NotNull Integer unionEnterpriseAuthId) {
        unionEnterpriseAuthAppService.deleteUnionEnterpriseAuthAppByUnionEnterpriseAuthId(unionEnterpriseAuthId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> relieveTask(String param) {
        List<TabSysUnionEnterpriseAuth> list = unionEnterpriseAuthService.listAll();
        if (CollectionUtils.isNotEmpty(list)) {
            List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterprise();
            Map<String, String> enterpriseMap = enterpriseList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId().toString(), e -> e.getEnterpriseName()));

            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (TabSysUnionEnterpriseAuth temp : list) {
                //待解除状态会进行轮训
                if (temp.getRelieveTime() != null && temp.getRelieveEffectDate() != null &&
                        UnionEnterpriseAuthStatusEnum.TO_DO_RELIEVE.getCode().equals(temp.getStatusFlag())) {
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTime(temp.getRelieveTime());
                    calendar.add(Calendar.DAY_OF_YEAR, temp.getRelieveEffectDate());

                    String[] smsArr;
                    //如果到达解除时间，发通知 boolean hasDate = sdf.format(calendar.getTime()).compareTo(sdf.format(new Date())) >= 0;
                    int days = differentDaysByMillisecond(calendar.getTime(), new Date());
                    if (days == 0) {

                        smsArr = new String[] { enterpriseMap.get(temp.getUnionEnterpriseId().toString()),
                                enterpriseMap.get(temp.getOwnEnterpriseId().toString()),
                                enterpriseMap.get(temp.getOwnEnterpriseId().toString())};

                        if (UnionEnterpriseAuthStatusEnum.TO_DO_RELIEVE.getCode().equals(temp.getStatusFlag())) {
                            //todo 发通知
                            //门店解除

                            storeAuthorizationApiService.cancelStoreAll(temp.getUnionEnterpriseId(), temp.getOwnEnterpriseId());

                            TransmitDataDTO transmitDataDTO = (TransmitDataDTO) JSON.parse(temp.getLogParams());
                            authorizationEnterConfApiService.delAuthorizationEnterpriseConf(temp.getUnionEnterpriseId(), temp.getOwnEnterpriseId(), transmitDataDTO);

                            userGroupPermissionApiService.relieveUnionEnterprise(temp.getUnionEnterpriseId(), temp.getOwnEnterpriseId());
                        }
                    } else {
                        //发送短信模板

                        smsArr = new String[] { enterpriseMap.get(temp.getUnionEnterpriseId().toString()),
                                enterpriseMap.get(temp.getOwnEnterpriseId().toString()),
                                String.valueOf(days),
                                sdf.format(calendar.getTime()),
                                enterpriseMap.get(temp.getOwnEnterpriseId().toString())};
                    }

                    ServiceResponse<UserDTO> userResponse = userApiService.getUserByEnterpriseId(temp.getUnionEnterpriseId());
                    //发送手机验证码
                    ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(days == 0 ? SMS_CODE_RELIEVED : SMS_CODE_RELIEVE,
                            temp.getUnionEnterpriseId(),
                            userResponse.getResult().getPhoneAreaCode(),
                            userResponse.getResult().getPhoneNumber(),
                            smsArr);
                    if (!smsSendResult.isSuccess()) {
                        LOGGER.warn("解除联合发送短信错误信息:{}", JSON.toJSONString(smsSendResult));
                    }
                }
            }
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<OwnEnterpriseAuthListDTO>> listOwnEnterpriseAuth(@NotNull Integer unionEnterpriseId, String startTime, String endTime) {
        List<OwnEnterpriseAuthListDTO> list = unionEnterpriseAuthService.listOwnEnterpriseAuth(unionEnterpriseId, startTime, endTime);
        if (CollectionUtils.isNotEmpty(list)) {
            //组装数据
            //1:自有商户数据
            List<Integer> enterpriseIdList = list.stream().mapToInt(e -> e.getEnterpriseId()).boxed().collect(Collectors.toList());
            Map<Integer, TabEnterprise> enterpriseMap = getEnterprise(enterpriseIdList);
            //2:超管数据
            Map<Integer, UserDTO> userMap = getUser(enterpriseIdList);
            //填充数据
            for (OwnEnterpriseAuthListDTO obj : list) {
                TabEnterprise enterprise = enterpriseMap.get(obj.getEnterpriseId());
                if (enterprise != null) {
                    obj.setCompanyName(enterprise.getCompanyName());
                    obj.setAddress(enterprise.getAddress());
                    obj.setEnterpriseName(enterprise.getEnterpriseName());
                    obj.setLogo(enterprise.getLogo());
                }
                UserDTO user = userMap.get(obj.getEnterpriseId());
                if (user != null) {
                    obj.setUserName(user.getUserName());
                    obj.setPhoneAreaCode(user.getPhoneAreaCode());
                    obj.setPhoneNumber(user.getPhoneNumber());
                }
            }
            return ServiceResponse.success(list);
        }
        return ServiceResponse.success(Collections.emptyList());
    }

    @Override
    public ServiceResponse<Page<UnionEnterpriseAuthResLogDTO>> listUnionEnterpriseAuthResLog(UnionEnterpriseAuthResLogQO param) {
        com.github.pagehelper.Page page = unionEnterpriseAuthResLogService.listUnionEnterpriseAuthResLog(param);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, UnionEnterpriseAuthResLogDTO.class));
    }

    @Override
    public ServiceResponse<Boolean> hasUnionEnterpriseAuth(Integer resourceGroupId) {
        return ServiceResponse.success(unionEnterpriseAuthService.hasUnionEnterpriseAuth(resourceGroupId));
    }

    private Map<Integer, TabEnterprise> getEnterprise(List<Integer> enterpriseIdList) {
        List<TabEnterprise> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIdList);
        if (CollectionUtils.isNotEmpty(enterpriseList)) {
            return enterpriseList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId(), e -> e));
        }
        return new HashMap<>();
    }

    private Map<Integer, UserDTO> getUser(List<Integer> enterpriseIdList) {
        ServiceResponse<List<UserDTO>> userResponse = userApiService.listUserByEnterpriseId(enterpriseIdList);
        if (userResponse.isSuccess()) {
            List<UserDTO> userList = userResponse.getResult();
            if (CollectionUtils.isNotEmpty(userList)) {
                return userList.stream().collect(Collectors.toMap(e -> e.getEnterpriseId(), e -> e));
            }
        }
        return new HashMap<>();
    }

    private TabSysUnionEnterpriseAuth checkExistUnionEnterprise(Integer ownEnterpriseId, Integer unionEnterpriseId) {
        TabSysUnionEnterpriseAuth old = unionEnterpriseAuthService.getUnionEnterprise(ownEnterpriseId, unionEnterpriseId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "联合商户数据不存在");
        }
        return old;
    }

    private TabSysUnionEnterpriseAuthApp checkUnionEnterpriseAuthApp(Integer unionEnterpriseAuthAppId) {
        TabSysUnionEnterpriseAuthApp old = unionEnterpriseAuthAppService.getUnionEnterpriseAuthApp(unionEnterpriseAuthAppId);
        if (old == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "策略不存在");
        }
        return old;
    }

    private void checkResourceDealing(Integer unionEnterpriseAuthId) {
        List<TabSysUnionEnterpriseAuthResDetail> list = unionEnterpriseAuthResDetailService.listUnionEnterpriseAuthResDetail(unionEnterpriseAuthId, null);
        if (CollectionUtils.isNotEmpty(list)) {
            if (list.stream().anyMatch(e -> UnionEnterpriseAuthDetailStatusEnum.RELIEVING.getCode().equals(e.getAuthStatus())
                    || UnionEnterpriseAuthDetailStatusEnum.RESOURCE_DEALING.getCode().equals(e.getAuthStatus()))) {
                throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "当前存在“资源处理中”或“资源解除中”，不允许同步");
            }
        }
    }

    private ResourceGroupDTO getResourceGroup(Integer resourceGroupId) {
        ServiceResponse<ResourceGroupDTO> resourceResponse = resourceGroupApiService.getResourceGroup(resourceGroupId);
        if (!resourceResponse.isSuccess() || resourceResponse.getResult() == null) {
            throw new CommonException(ErrorCode.PARAMETER_ERROR.getCode(), "资源组数据不存在");
        }
        return resourceResponse.getResult();
    }

    /**
     * 通过时间秒毫秒数判断两个时间的间隔
     * @param date1
     * @param date2
     * @return
     */
    private static int differentDaysByMillisecond(Date date1, Date date2) {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        long startDateTime = 0;
        long endDateTime = 0;

        try {
            startDateTime = dateFormat.parse(dateFormat.format(date1)).getTime();
            endDateTime= dateFormat.parse(dateFormat.format(date2)).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return (int) ((endDateTime - startDateTime) / (1000 * 3600 * 24));
    }

    private void saveRefer(Integer businessId, String businessName, Integer resourceGroupId) {
        PopupDTO popupDTO = new PopupDTO();
        popupDTO.setBusinessId(String.valueOf(businessId));
        popupDTO.setBusinessName(businessName);
        popupDTO.setBusinessType(REFER_UNION_ENTERPRISE);

        popupDTO.setRelationBusinessIdList(new RelationBusinessInfoDTO(REFER_RESOURCE, String.valueOf(resourceGroupId)));
        ServiceResponse response = PopupUtil.savePopupLog(popupDTO);
        LOGGER.info("联合商户保存数据引用结果：{}", JSON.toJSONString(response));
    }

}
